@@ -34,11 +34,15 @@ class Field(_BaseSubject):
         can be null (blank). Defaults to False, which will cause errors to
         be raised when checking empty strings.
     :param rules: a list of rules to apply to this field during checks.
+    :param strip: (optional) apply pre-processing to values in the field which
+        applies the str.strip method, removing leading and trailing whitespaces,
+        prior to checking rules.
     """
 
-    def __init__(self, nullable=False, rules: list = None):
+    def __init__(self, nullable=False, rules: list = None, **kwargs):
         super().__init__(rules)
         self.nullable = nullable
+        self.strip = kwargs.get('strip')
 
         if not self.nullable:
             self.rules.append(clr.NotNull())
@@ -72,7 +76,7 @@ class Field(_BaseSubject):
         errors = self._check(values, rule_type=cr.Rule, **kwargs)
         assert not errors, str(errors)
 
-    def _check(self, data, cix=-1, rule_type=None, **kwargs) -> Union[ex.CellError, None]:
+    def _check(self, data, cix=-1, rule_type=None, **kwargs) -> Union[ex.CellError, ex.ColumnError, None]:
         """
         Check data against field rules of specified rule type
 
@@ -89,12 +93,15 @@ class Field(_BaseSubject):
         """
 
         # if data is nullable and value is empty, skip all checks
-        if self.nullable and issubclass(rule_type, clr.Rule) and data == '':
+        if self.nullable and rule_type == clr.Rule and data == '':
             pass
         else:
-            e = super()._check(data, rule_type=rule_type)
+            e = super()._check(data, rule_type=rule_type, strip=self.strip)
             if e:
-                return ex.CellError(cix, errors=e, **kwargs)
+                if rule_type == cr.Rule:
+                    return ex.ColumnError(cix, errors=e, **kwargs)
+                else:
+                    return ex.CellError(cix, errors=e, **kwargs)
 
     def _comparison_columns(self) -> set:
         """
