@@ -6,13 +6,15 @@ This submodule contains the File class, and it's closely related Layout class.
 
 import csv
 from pathlib import Path
-from typing import Union, Dict, List
+from typing import Union, Dict, List, Iterable
 
 from rumydata import exception as ex
 from rumydata import field
 from rumydata._base import _BaseSubject
 from rumydata.rules import table, column as cr, header as hr, row as rr, cell as clr
 
+__all__ = ['Layout', 'CsvFile', 'ExcelFile']
+
 
 class Layout(_BaseSubject):
     """
@@ -29,6 +31,10 @@ class Layout(_BaseSubject):
     :param _definition: dictionary of column names with DataType definitions
     :param skip_header: (optional) a boolean control to skip validation of
         the header in the file. Defaults to False.
+    :param header_mode: (optional) sets the mode for checking the header.
+        Defaults to 'exact', but also accepts 'startswith' and 'contains'. This
+        allows optional partial matching of header values in a layout to the
+        values obtained in tabular data.
     :param empty_row_ok: (optional) a boolean control to skip validation of
         any row that is completely empty (i.e. every field is blank).
         Defaults to False.
@@ -39,6 +45,11 @@ class Layout(_BaseSubject):
     def __init__(self, _definition: Dict[str, field.Field], **kwargs):
         self.skip_header = kwargs.pop('skip_header', False)
         self.empty_row_ok = kwargs.pop('empty_row_ok', False)
+        self.header_mode = kwargs.pop('header_mode', 'exact')
+
+        header_modes = ('exact', 'startswith', 'contains')
+        if self.header_mode not in header_modes:
+            raise ValueError(f"header_mode argument invalid. Must be one of: {header_modes}")
 
         super().__init__(**kwargs)
 
@@ -55,15 +66,23 @@ class Layout(_BaseSubject):
             rr.RowLengthGTE(self.field_count)
         ])
 
-    def documentation(self):
+    def documentation(self, doc_type='md'):
         """
         Technical documentation
 
         Generates detailed specification of the defined layout.
 
+        :param doc_type: format of returned technical documentation
+
         :return: a Markdown formatted string describing the layout
         """
-        return self._markdown_digest()
+        if doc_type == 'md':
+            return self._markdown_digest()
+        elif doc_type == 'html':
+            import markdown
+            return markdown.markdown(self._markdown_digest(), tab_length=2)
+        else:
+            raise TypeError(f"Invalid format type: {doc_type}")
 
     def check_header(self, row: List[str], rix=0):
         """
@@ -110,9 +129,6 @@ class Layout(_BaseSubject):
         ])
         return fields
 
-    def _digest(self):
-        return [[f'Name: {k}', *v._digest()] for k, v in self.layout.items()]
-
     def _check(self, row, rule_type, rix=None) -> Union[ex.UrNotMyDataError, None]:
         if rule_type == hr.Rule and self.skip_header:
             return
@@ -142,27 +158,10 @@ class Layout(_BaseSubject):
             if e:
                 return ex.RowError(rix, errors=e)
 
-    def _comparison_columns(self) -> set:
-        """
-        Comparison fields report
 
-        A method to report all columns that will need to be compared while
-        checking rules. This works by checking the equivalent method for each
-        field in the layout. This makes it convenient to know which values in
-        a row need to be stored in a dictionary for possible comparison in each
-        cell.
-
-        :return: a set of the columns that will need to be compared.
-        """
-        compares = set()
-        for v in self.layout.values():
-            compares.update(v._comparison_columns())
-        return compares
-
-
-class File(_BaseSubject):
+class _BaseFile(_BaseSubject):
     """
-    File class
+    Base File class
 
     This class provides a way to validate the contents of a file against a
     Layout, and report any rule violations that exist. This is the primary
@@ -171,48 +170,25 @@ class File(_BaseSubject):
     :param layout: a Layout object which defines the fields that make up the
         data set, along with the various rules that should be applied to
         each one.
-    :param skip_rows: (optional) the number of rows to skip before starting
-        evaluation. Defaults to 0.
-    :param max_errors: (optional) the maximum number of row errors to be
+    :param skip_rows: the number of rows to skip before starting evaluation.
+    :param max_errors: the maximum number of row errors to be
         collected before halting validation of rows and raising a FileError.
         This is used to prevent overly verbose (and mostly useless)
-        validation reports from being generated. Defaults to 100. The error
-        limit can overwritten (set to unlimited) by providing a value of -1.
-    :param file_type: (optional) the type of file to be evaluated. Valid
-        values are ['csv', 'excel']. Defaults to 'csv'.
-    :param dialect: (optional) only valid with csv file type. Controls csv
-        dialect parsing.
-    :param delimiter: (optional) only valid with csv file type. Controls csv
-        delimiter parsing.
-    :param quotechar: (optional) only valid with csv file type. Controls csv
-        quote character parsing.
-    :param sheet: (optional) only valid with excel file type. Determines
-        which sheet will be read for tabular data evaluation.
+        validation reports from being generated. The error limit can be set to
+        unlimited by providing a value of -1.
     """
 
-    def __init__(self, layout: Union[Layout, Dict], **kwargs):
-        self.max_errors = kwargs.pop('max_errors', 100)
-        self.file_type = kwargs.pop('file_type', 'csv')
-
-        if self.file_type == 'csv':
-            x = {x: kwargs.pop(x, None) for x in ['dialect', 'delimiter', 'quotechar']}
-            self.csv_kwargs = {k: v for k, v in x.items() if v}
-        elif self.file_type == 'excel':
-            x = {x: kwargs.pop(x, None) for x in ['sheet']}
-            self.excel_kwargs = {k: v for k, v in x.items() if v}
-        else:
-            raise Exception(f'Invalid file type: {self.file_type}')
-
-        self.skip_rows = kwargs.pop('skip_rows', 0)
+    def __init__(self, layout: Layout, skip_rows=0, max_errors=100, **kwargs):
+        self.skip_rows = skip_rows
+        self.max_errors = max_errors
 
         super().__init__(**kwargs)
-
         self.layout = Layout(layout) if isinstance(layout, Dict) else layout
         self.rules.extend([
             table.FileExists()
         ])
 
-    def check(self, file_path: Union[str, Path]) -> None:
+    def check(self, file_path: Union[str, Path], doc_type: str = None):
         """
         File check method
 
@@ -220,15 +196,41 @@ class File(_BaseSubject):
 
         :param file_path: a file path provided as a string, or a pathlib Path
             object.
+        :param doc_type: the type of output to return with exception details,
+            rather than raising an exception. Valid options are ['md', 'html']
         """
-        errors = self._check(file_path)
-        assert not errors, str(errors)
+        errors = self._check(Path(file_path))
+        try:
+            assert not errors, str(errors)
+            msg = f"Validated file successfully: {Path(file_path).as_posix()}"
+        except AssertionError as ae:
+            if not doc_type:
+                raise ae
+            else:
+                msg = str(ae)
+
+        if not doc_type:
+            return
+        elif doc_type == 'md':
+            return msg
+        elif doc_type == 'html':
+            import markdown
+            return markdown.markdown(msg, tab_length=2)
+        else:
+            raise TypeError(f"Invalid doc type: {doc_type}")
+
+    def _rows(self, file: Path):
+        pass
+
+    @staticmethod
+    def _row_handler(row: list) -> List[str]:
+        return row
 
     def _check(self, filepath: Union[str, Path], **kwargs) -> Union[ex.FileError, None]:
         p = Path(filepath) if isinstance(filepath, str) else filepath
         e = super()._check(p, rule_type=table.Rule)  # check files-based rules first
         if e:
-            return ex.FileError(file=filepath, errors=e)
+            return ex.FileError(file=p.as_posix(), errors=e)
 
         column_cache = {
             k: [] for k, v in self.layout.layout.items()
@@ -239,26 +241,12 @@ class File(_BaseSubject):
             for k in column_cache.keys()
         }
 
-        with open(p, newline='') as f:
-            if self.file_type == 'csv':
-                generator = csv.reader(f, **self.csv_kwargs)
-
-                def row_handler(r):  # handle csv rows as-is
-                    return r
-            elif self.file_type == 'excel':
-                from openpyxl import load_workbook
-                wb = load_workbook(p)
-                sheet = self.excel_kwargs.get('sheet')
-                ws = wb.get_sheet_by_name(sheet) if sheet else wb.active
-                generator = ws.values
-
-                def row_handler(r):  # convert values to strings and replace None with empty
-                    return [str(x or '') for x in r]
-
+        max_error_rule = table.MaxError(self.max_errors)
+        with self._rows(p) as generator:
             for rix, row in enumerate(generator):
                 if rix < self.skip_rows:
                     continue
-                row = row_handler(row)
+                row = self._row_handler(row)
                 rt = hr.Rule if rix == (0 + self.skip_rows) else rr.Rule
                 re = self.layout._check(row, rule_type=rt, rix=rix)
                 if re:
@@ -266,16 +254,117 @@ class File(_BaseSubject):
                     if rix == 0:  # if header error present, stop checking rows
                         break
                     if len(e) > self.max_errors:
-                        m = f"max of {str(self.max_errors)} row errors exceeded"
-                        e.append(ex.MaxExceededError(m))
+                        e.append(max_error_rule._exception_msg())
                         break
                 if rix > 0:
                     for k, ix in column_cache_map.items():
                         column_cache[k].append(row[ix])
 
-        for k, v in column_cache.items():
-            ce = self.layout.layout[k]._check(v, rule_type=cr.Rule)
-            if ce:
-                e.append(ce)
-        if e:
-            return ex.FileError(file=p.name, errors=e)
+            for k, v in column_cache.items():
+                ce = self.layout.layout[k]._check(
+                    v, cix=column_cache_map[k], rule_type=cr.Rule, name=k
+                )
+                if ce:
+                    e.append(ce)
+            if e:
+                return ex.FileError(file=p.name, errors=e)
+
+
+class CsvFile(_BaseFile):
+    """
+    CSV File class
+
+    This class provides a way to validate the contents of a file against a
+    Layout, and report any rule violations that exist. This is the primary
+    means of using this package.
+
+    :param layout: a Layout object which defines the fields that make up the
+        data set, along with the various rules that should be applied to
+        each one.
+    :param skip_rows: the number of rows to skip before starting evaluation.
+    :param max_errors: (optional) the maximum number of row errors to be
+        collected before halting validation of rows and raising a FileError.
+        This is used to prevent overly verbose (and mostly useless)
+        validation reports from being generated. Defaults to 100. The error
+        limit can overwritten (set to unlimited) by providing a value of -1.
+    :param dialect: (optional) Controls csv dialect parsing.
+    :param delimiter: (optional) Controls csv delimiter parsing.
+    :param quotechar: (optional) Controls csv quote character parsing.
+    """
+
+    def __init__(self, layout: Union[Layout, dict], skip_rows=0, max_errors=100, **kwargs):
+        x = {x: kwargs.pop(x, None) for x in ['dialect', 'delimiter', 'quotechar']}
+        self.csv_kwargs = {k: v for k, v in x.items() if v}
+        super().__init__(layout, skip_rows, max_errors, **kwargs)
+
+    def _rows(self, file: Path):
+        class Handler:
+            def __init__(self, file_path: Path, **kwargs):
+                self.file_path = file_path
+                self.csv_kwargs = kwargs
+
+            def __enter__(self) -> Iterable:
+                self.file_object = self.file_path.open(newline='')
+                return csv.reader(self.file_object, **self.csv_kwargs)
+
+            def __exit__(self, exc_type, exc_val, exc_tb):
+                self.file_object.close()
+
+        return Handler(file, **self.csv_kwargs)
+
+
+class ExcelFile(_BaseFile):
+    """
+    Excel File class
+
+    This class provides a way to validate the contents of an Excel file against a
+    Layout, and report any rule violations that exist. This is the primary
+    means of using this package.
+
+    :param layout: a Layout object which defines the fields that make up the
+        data set, along with the various rules that should be applied to
+        each one.
+    :param skip_rows: the number of rows to skip before starting evaluation.
+    :param max_errors: the maximum number of row errors to be
+        collected before halting validation of rows and raising a FileError.
+        This is used to prevent overly verbose (and mostly useless)
+        validation reports from being generated. The error
+        limit can be set to unlimited by providing a value of -1.
+
+    """
+
+    def __init__(self, layout: Union[Layout, Dict], skip_rows=0, max_errors=100, **kwargs):
+        try:
+            __import__('openpyxl')
+        except ModuleNotFoundError:
+            raise ModuleNotFoundError(
+                "openpyxl not available for import. You must install this"
+                " package before you can use the ExcelFile class."
+            )
+
+        x = {x: kwargs.pop(x, None) for x in ['sheet']}
+        self.excel_kwargs = {k: v for k, v in x.items() if v}
+
+        super().__init__(layout, skip_rows, max_errors, **kwargs)
+
+    def _rows(self, file: Path, sheet=None):
+        class Handler:
+            def __init__(self, file_path, **kwargs):
+                self.file_path = file_path
+                self.excel_kwargs = kwargs
+
+            def __enter__(self) -> Iterable:
+                from openpyxl import load_workbook
+                wb = load_workbook(file)
+                sheet_name = self.excel_kwargs.get('sheet')
+                ws = wb.get_sheet_by_name(sheet_name) if sheet else wb.active
+                return ws.values
+
+            def __exit__(self, exc_type, exc_val, exc_tb):
+                pass
+
+        return Handler(file, **self.excel_kwargs)
+
+    @staticmethod
+    def _row_handler(row: list) -> List[str]:
+        return [str(x or '') for x in row]
